@@ -1,6 +1,6 @@
 """多维多时间序列神经网络滚动训练的数据工具.
 
-version: 0.0.7
+version: 0.0.11
 
 author: Congyu Wang
 
@@ -32,7 +32,8 @@ class TimeSeriesData:
         Args:
             dates: 日期列, 1D ``numpy.ndarray``, 整数
             data: 训练输入的X，2D ``numpy.ndarray``, (日期长度 x 特征数量)
-            labels: 训练标签Y, 1D ``numpy.ndarray``, 长度与dates相同
+            labels: 训练标签Y, 1D ``numpy.ndarray``, 长度与dates相同。
+                如果为分类问题则是2D, (日期长度 x 类别数量)
 
         """
         # 检查参数类型
@@ -44,7 +45,7 @@ class TimeSeriesData:
         if len(dates) != len(data) or len(dates) != len(labels):
             raise ValueError("Bad data shape")
         # 检查维度是否正确
-        if dates.ndim != 1 or data.ndim != 2 or labels.ndim != 1:
+        if dates.ndim != 1 or data.ndim != 2 or not 1 <= labels.ndim <= 2:
             raise ValueError("Wrong dimensions")
         self.dates = dates.astype(_np.int32)
         self.data = data
@@ -135,6 +136,24 @@ class TrainValData:
                 raise ValueError("time series do not have "
                                  "the same number of features")
 
+        # 确保标签维度一致
+        label_dims = time_series_list[0].labels.ndim
+        for series in time_series_list:
+            if series.labels.ndim != label_dims:
+                raise ValueError("time labels do not have "
+                                 "the same number of dimensions")
+
+        # 标签类别数量
+        class_num = 0
+        if label_dims == 2:
+            class_num = time_series_list[0].labels.shape[1]
+            for series in time_series_list:
+                if series.labels.shape[1] != class_num:
+                    raise ValueError("time series labels do not have "
+                                     "the same number of classes")
+
+        self.__class_num = class_num
+
         # 获取日期列表（所有时间序列日期的并集）
         self.__distinct_dates = _np.unique([date for stock in time_series_list
                                             for date in stock.dates])
@@ -145,21 +164,35 @@ class TrainValData:
         self.__data = _np.empty((len(time_series_list),
                                  len(self.__distinct_dates),
                                  self.__feature_counts))
-        self.__labels = _np.empty((len(time_series_list),
-                                   len(self.__distinct_dates)))
+        if self.__class_num == 0:
+            self.__labels = _np.empty((len(time_series_list),
+                                       len(self.__distinct_dates)))
+        else:
+            self.__labels = _np.empty((len(time_series_list),
+                                       len(self.__distinct_dates),
+                                       self.__class_num))
+
+        self.__series_date_matrix = _np.empty((len(time_series_list),
+                                               len(self.__distinct_dates), 2))
         self.__data[:] = fill_na
         self.__labels[:] = fill_na
+        self.__series_date_matrix[:] = fill_na
 
         # 根据日期序列的位置向张量填充数据
         dates_positions = {date: index
                            for index, date in enumerate(self.__distinct_dates)}
         dates_position_mapper = _np.vectorize(lambda d: dates_positions[d])
         for i, series in enumerate(time_series_list):
-            # 找到该序列series.dates日期在日期列表中的位置，
+            # 找到该序列series.dates日期在日期列表中的位置
             # 将第i个序列填充至tensor的第i行
             position_index = dates_position_mapper(series.dates)
             self.__data[i, position_index, :] = series.data
-            self.__labels[i, position_index] = series.labels
+            if self.__class_num == 0:
+                self.__labels[i, position_index] = series.labels
+            else:
+                self.__labels[i, position_index, :] = series.labels
+            self.__series_date_matrix[i, position_index, 0] = series.dates
+            self.__series_date_matrix[i, position_index, 1] = i
 
         # numpy -> tensor
         self.__data = _tf.constant(self.__data, dtype=_tf.float32)
@@ -173,7 +206,6 @@ class TrainValData:
     def get(self,
             start_date: int,
             order="by_date",
-            mode="in_memory",
             validate_only=False):
         """获取从某天开始的训练集和验证集.
 
@@ -193,11 +225,6 @@ class TrainValData:
             start_date: 该轮训练开始日期，整数``YYYYMMDD``
             order: 有三种顺序 ``shuffle``, ``by_date``, ``by_series``。
                 分别为随机打乱股票和时间，按时间顺序优先，按股票顺序优先，默认by_date。
-            mode: `generator` 或 `in_memory`. generator 速度极慢，
-                in_memory速度较快，默认in_memory。feature、series数量大显存不足时
-                可以使用generator。
-                'in_memory'模式股票数量较大、history_length较大、或者step较小时，
-                可能会要求较大显卡内存。
             validate_only: 如果设置为True，则只返回validate set
                 和训练集、验证集时间信息。可以用于训练后的分析。
 
@@ -210,17 +237,9 @@ class TrainValData:
             ValueError: 日期范围超出最大日期会报ValueError。
 
         """
-        if mode == "generator":
-            return self.__get_from_generator__(start_date,
-                                               order,
-                                               validate_only)
-        elif mode == "in_memory":
-            return self.__get_in_memory__(start_date,
-                                          order,
-                                          validate_only)
-        else:
-            raise ValueError("mode unimplemented, choose from `generator` "
-                             "and `in_memory`")
+        return self.__get_in_memory__(start_date,
+                                      order,
+                                      validate_only)
 
     def __get_in_memory__(self,
                           start_date,
@@ -236,44 +255,31 @@ class TrainValData:
         train_args, val_args, dates_info = self.__get_period_info__(start_date,
                                                                     order)
         # 将输入的数据、标签片段转化为单个sample包含history日期长度的历史信息
-        val_x, val_y = __full_tensor_generation__(*val_args)
+        (val_x,
+         val_y,
+         val_dates_series) = __full_tensor_generation__(*val_args)
         # 转化为tensorflow DataSet
         val = _tf.data.Dataset.from_tensor_slices((val_x, val_y))
+        val_dates_series = val_dates_series.numpy().astype(int)
+        val_dates_list = val_dates_series[:, 0].tolist()
+        val_series_list = val_dates_series[:, 1].tolist()
+        dates_info["validation"]["dates_list"] = val_dates_list
+        dates_info["validation"]["series_list"] = val_series_list
 
         if validate_only:
             return val, dates_info
 
-        train_x, train_y = __full_tensor_generation__(*train_args)
+        (train_x,
+         train_y,
+         train_dates_series) = __full_tensor_generation__(*train_args)
         train = _tf.data.Dataset.from_tensor_slices((train_x, train_y))
+        train_dates_series = train_dates_series.numpy().astype(int)
+        train_dates_list = train_dates_series[:, 0].tolist()
+        train_series_list = train_dates_series[:, 1].tolist()
+        dates_info["training"]["dates_list"] = train_dates_list
+        dates_info["training"]["series_list"] = train_series_list
         return train, val, dates_info
 
-    def __get_from_generator__(self,
-                               start_date,
-                               order="by_date",
-                               validate_only=False):
-        """使用tensorflow.data.DataSet.from_generator，占用内存少，生成数据慢."""
-        # 获取用于构建训练集、验证集的相关信息
-        train_args, val_args, dates_info = self.__get_period_info__(start_date,
-                                                                    order)
-        shapes = (_tf.TensorShape((self.__history_length,
-                                   self.__feature_counts)),
-                  _tf.TensorShape(()))
-        types = (_tf.float32, _tf.float32)
-        val_dataset = _tf.data.Dataset.from_generator(__generator__,
-                                                      args=val_args,
-                                                      output_shapes=shapes,
-                                                      output_types=types)
-
-        if validate_only:
-            return val_dataset, dates_info
-
-        train_dataset = _tf.data.Dataset.from_generator(__generator__,
-                                                        args=train_args,
-                                                        output_shapes=shapes,
-                                                        output_types=types)
-
-        return train_dataset, val_dataset, dates_info
-
     def __get_period_info__(self, start_date, order="by_date"):
         """根据开始时间计算用于构建训练集、验证集的相关信息."""
         if type(start_date) is not int:
@@ -290,11 +296,11 @@ class TrainValData:
             raise ValueError("date range exceeded end of dates")
 
         # 计算各个时间节点在时间列表中的位置
-        # 训练集开始位置
+        # 训练集开始位置(data的开始位置)
         train_start_index = __first_index__(self.__distinct_dates, first_date)
         # 训练集结束位置(不包含)
         train_end_index = train_start_index + self.__train_length
-        # 验证集开始位置
+        # 验证集开始位置(data的开始位置)
         val_start_index = (train_end_index -
                            self.__history_length +
                            self.__train_val_gap + 1)
@@ -338,6 +344,7 @@ class TrainValData:
         length = end_index - start_index
         data = self.__data[:, start_index:end_index, :]
         label = self.__labels[:, start_index:end_index]
+        dates_series = self.__series_date_matrix[:, start_index:end_index, :]
         generation_list = [[series_i, t]
                            for t in range(0,
                                           length - self.__history_length + 1,
@@ -357,7 +364,7 @@ class TrainValData:
         generation_list = _tf.constant(generation_list, dtype=_tf.int32)
         history_length = _tf.constant(self.__history_length, dtype=_tf.int32)
 
-        return data, label, generation_list, history_length
+        return data, label, generation_list, history_length, dates_series
 
     def __dates_info__(self,
                        train_start_index,
@@ -396,7 +403,7 @@ class TrainValData:
             },
             "validation": {
                 "start_date": int(self.__distinct_dates[validation_beginning]),
-                "end_date": int(self.__distinct_dates[validation_ending])
+                "end_date": int(self.__distinct_dates[validation_ending]),
             }
         }
         return dates_info
@@ -408,46 +415,37 @@ def __history_expander__(data_tensor, history_length):
     data_expanded = _tf.stack([data_tensor[:, i: (total_time_length + i
                                                   - history_length + 1), :]
                                for i in _tf.range(history_length)])
-    # 调整维度顺序 (series, time, history, features)
+    # 调整维度顺序 (history, sample)
     return _tf.transpose(data_expanded, perm=[1, 2, 0, 3])
 
 
-def __full_tensor_generation__(data, label, generation_list, history):
+def __full_tensor_generation__(data,
+                               label,
+                               generation_list,
+                               history,
+                               dates_series):
     """将输入的数据、标签片段转化为单个sample包含history日期长度的历史信息."""
     # 先将该数据片段的历史维度展开
-    data_expanded = __history_expander__(data, history)
 
     # 根据generation_list指定的series，日期，获取标签及数据片段
+    total_time_length = data.shape[1]
+    expanded = [_tf.gather_nd(data[:, i: (total_time_length + i
+                                          - history + 1), :], generation_list)
+                for i in _tf.range(history)]
+    data_all = _tf.stack(expanded, axis=1)
     label_all = _tf.gather_nd(label[:, history - 1:], generation_list)
-    data_all = _tf.gather_nd(data_expanded, generation_list)
+    dates_series_all = _tf.gather_nd(dates_series[:, history - 1:],
+                                     generation_list)
 
     # 去掉所有包含缺失数据的某股票某时间历史片段
-    label_nan = _tf.cast(_tf.math.is_nan(label_all), _tf.int64)
-    data_nan = _tf.cast(_tf.math.is_nan(data_all), _tf.int64)
-    nan_series_time_index = _tf.reduce_sum(_tf.reduce_sum(data_nan, axis=2), axis=1)
-    not_nan = _tf.add(nan_series_time_index, label_nan) == 0
-    return data_all[not_nan], label_all[not_nan]
-
-
-def __training_example_getter__(data, label, series_i, i, history_length):
-    """``DataSet.from_generator``会用到的函数，获取单个训练数据."""
-    x = data[series_i][i: i + history_length]
-    y = label[series_i][i + history_length - 1]
-
-    if (_tf.reduce_sum(_tf.cast(_tf.math.is_nan(x), _tf.int64)) == 0 and
-            _tf.reduce_sum(_tf.cast(_tf.math.is_nan(y), _tf.int64)) == 0):
-        return x, y
-    else:
-        return None
-
-
-def __generator__(data, label, generation_list, history_length):
-    """``DataSet.from_generator``会用到的generator."""
-    for series_i, i in generation_list:
-        d = __training_example_getter__(data, label, series_i, i, history_length)
-        if d:
-            x, y = d
-            yield x, y
+    label_nan = _tf.math.is_nan(label_all)
+    data_nan = _tf.math.is_nan(data_all)
+    nan_series_time_index = _tf.math.reduce_any(
+        _tf.math.reduce_any(data_nan, axis=2),
+        axis=1
+    )
+    not_nan = _tf.math.logical_not(_tf.math.logical_or(nan_series_time_index, label_nan))
+    return data_all[not_nan], label_all[not_nan], dates_series_all[not_nan]
 
 
 def __first_index__(array, element):
